@@ -1130,6 +1130,13 @@ class MBIODevice(object):
         self.logger.error('%s: reboot not implemented!')
         return False
 
+    def isUpgradePossible(self):
+        return False
+
+    def upgrade(self):
+        self.logger.error('%s: upgrade not implemented!')
+        return False
+
 
 class MBIODeviceGeneric(MBIODevice):
     def onInit(self):
