@@ -599,6 +599,13 @@ class MBIODeviceDigimatSIO(MBIODevice):
             return True
         return False
 
+    def isUpgradePossible(self):
+        return True
+
+    def upgrade(self):
+        s=self.mbio.sioscanner()
+        return s.upgrade(self.gateway.host)
+
 
 class SIOConfigurator(object):
     def __init__(self, host, token=None, logger=None):
