@@ -89,7 +89,7 @@ class MBIONetworkScanner(object):
                 break
 
             try:
-                self.logger.debug('%s: scanning host %s...' % (self.__class__.__name__, host))
+                # self.logger.debug('%s: scanning host %s...' % (self.__class__.__name__, host))
                 self.microsleep()
                 hostid=self.probe(host)
                 if hostid:
@@ -101,7 +101,7 @@ class MBIONetworkScanner(object):
             # we must emit a task_done() for every queued element
             self._queue.task_done()
 
-        self.logger.debug('%s: exit scanning thread' % (self.__class__.__name__))
+        # self.logger.debug('%s: exit scanning thread' % (self.__class__.__name__))
 
     def scan(self):
         self.logger.info('%s: scanning network...' % self.__class__)
