@@ -51,6 +51,7 @@ class MBIOValue(object):
         self._name=name
         self._key=parent.computeValueKeyFromName(name)
         self._writable=writable
+        self._marked=False
         self._enable=True
         if type(unit)==str:
             unit=parent.units.getByName(unit)
@@ -306,6 +307,17 @@ class MBIOValue(object):
     def disable(self):
         self.enable(False)
 
+    def isMarked(self):
+        return self._marked
+
+    def mark(self, state=True):
+        if self._marked!=state:
+            self._marked=state
+            self.updateFlags()
+
+    def unmark(self):
+        self.mark(False)
+
     def age(self):
         return time.time()-self._stamp
 
@@ -425,6 +437,8 @@ class MBIOValue(object):
             flags+='X'
         if self.isRemoteManual():
             flags+='R'
+        if self.isMarked():
+            flags+='K'
 
         if self.isWritable():
             flags+='W'
@@ -980,6 +994,13 @@ class MBIOValues(Items):
         for value in self:
             value.auto()
 
+    def mark(self, state=True):
+        for value in self:
+            value.mark(state)
+
+    def unmark(self):
+        self.mark(False)
+
     # Force return None if called with an unknown attribute (value name)
     def __getattr__(self, name):
         try:
