@@ -218,6 +218,19 @@ class MBIOWsMonitor(MBIOTask):
                 return True
         self.cb_write_failure(handler)
 
+    def cb_upgradedevice(self, handler, params):
+        mbio=self.getMBIO()
+        gateway=params.get('gateway')
+        key=params.get('key')
+        g=mbio.gateway(gateway)
+        if g:
+            d=g.device(key)
+            if d:
+                d.upgrade()
+                self.cb_write_success(handler)
+                return True
+        self.cb_write_failure(handler)
+
     def cb_beep(self, handler, params):
         mbio=self.getMBIO()
         mbio.beep()
@@ -311,13 +324,46 @@ class MBIOWsMonitor(MBIOTask):
                 item['countvalues']=d.values.count()
                 item['ismanualvalue']=d.isManualValue()
                 item['countvalues']=d.values.count()
-                item['canreboot']=d.isRebootPossible()
+                item['reboot']=d.isRebootPossible()
+                item['upgrade']=d.isUpgradePossible()
                 items.append(item)
 
         data={'data': items}
         self.cb_write_json(handler, data)
         return True
 
+    def cb_getalldevices(self, handler, params):
+        mbio=self.getMBIO()
+        items=[]
+        for g in mbio.gateways.all():
+            if g is not None:
+                for d in g.devices.all():
+                    self.microsleep()
+                    item={'gateway': g.key, 'address': d.address, 'key': d.key, 'vendor': d.vendor, 'model': d.model, 'state': d.statestr()}
+                    item['class']=d.__class__.__name__
+                    item['version']=None
+                    if d.version and d.firmware:
+                        item['version']=f'{d.version}/{d.firmware}'
+                    elif d.version:
+                        item['version']=d.version
+                    else:
+                        item['version']=d.firmware
+
+                    item['statetime']=int(d.statetime())
+                    item['countmsg']=d.countMsg
+                    item['countmsgerr']=d.countMsgErr
+                    item['error']=d.isError()
+                    item['countvalues']=d.values.count()
+                    item['ismanualvalue']=d.isManualValue()
+                    item['countvalues']=d.values.count()
+                    item['reboot']=d.isRebootPossible()
+                    item['upgrade']=d.isUpgradePossible()
+                    items.append(item)
+
+        data={'data': items}
+        self.cb_write_json(handler, data)
+        return True
+
     def cb_gettaskvalues(self, handler, params):
         mbio=self.getMBIO()
         t=mbio.task(params.get('task'))
@@ -446,6 +492,7 @@ class MBIOWsMonitor(MBIOTask):
                     </div>
                     <div class="text-muted small">{mbio.version} /
                     <a href="/gateways">Gateways</a> /
+                    <a href="/alldevices">*Devices</a> /
                     <a href="/values">Values</a> /
                     <a href="/valuesmarked">Marks</a>
                     </div>
@@ -555,6 +602,7 @@ class MBIOWsMonitor(MBIOTask):
                         <p class="text-secondary mb-0"><b>Gateways</b></p>
                     </div>
                     <div class="text-muted small">{mbio.version} /
+                    <a href="/alldevices">*Devices</a> /
                     <a href="/tasks">Tasks</a> /
                     <a href="/values">Values</a> /
                     <a href="/valuesmarked">Marks</a>
@@ -679,6 +727,7 @@ class MBIOWsMonitor(MBIOTask):
             <p class="text-secondary mb-0"><a href="/gateways">Gateways</a> / <b>{gateway}</b> / devices</p>
             </div>
             <div class="text-muted small">{mbio.version} /
+            <a href="/alldevices">*Devices</a> /
             <a href="/tasks">Tasks</a> /
             <a href="/values">Values</a> /
             <a href="/valuesmarked">Marks</a>
@@ -692,6 +741,7 @@ class MBIOWsMonitor(MBIOTask):
             <th>Vendor</th>
             <th>Model</th>
             <th>Version</th>
+            <th>Upgrade</th>
             <th>State</th>
             <th>Age</th>
             <th>Error</th>
@@ -735,6 +785,20 @@ class MBIOWsMonitor(MBIOTask):
                 { data: "vendor" },
                 { data: "model" },
                 { data: "version" },
+                { data: "upgrade",
+                    render: function (data, type, row) {
+                        if (type !== "display") return data;
+                        if (row.upgrade) {
+                            return `<div class="d-flex justify-content-between align-items-center">
+                                    <button class="btn btn-sm btn-outline-secondary cell-action"
+                                        data-field="upgrade" data-id="${row.key}">
+                                    Upgrade
+                                    </button>
+                                    </div>`;
+                        }
+                        return '';
+                    }
+                },
                 { data: "state",
                     render: function (data, type, row) {
                         if (type !== "display") return data;
@@ -753,21 +817,20 @@ class MBIOWsMonitor(MBIOTask):
                 { data: "countmsg" },
                 { data: "countmsgerr" },
                 { data: "class" },
-                { data: "canreboot",
+                { data: "reboot",
                     render: function (data, type, row) {
                         if (type !== "display") return data;
-                        if (row.canreboot) {
+                        if (row.reboot) {
                             return `<div class="d-flex justify-content-between align-items-center">
-                                <button class="btn btn-sm btn-outline-secondary cell-action"
-                                    data-field="reboot" data-id="${row.key}">
-                                Reboot
-                                </button>
-                                </div>`;
+                                    <button class="btn btn-sm btn-outline-secondary cell-action"
+                                        data-field="reboot" data-id="${row.key}">
+                                    Reboot
+                                    </button>
+                                    </div>`;
                         }
-
                         return '';
                     }
-                },
+                }
             ],
             rowCallback: function (row, data) {
                 if (data.error) {
@@ -799,6 +862,11 @@ class MBIOWsMonitor(MBIOTask):
                         const r = await fetch(`/api/v1/rebootdevice?gateway=${row.gateway}&key=${row.key}`, {method: "GET"});
                     } catch (err) {
                     }
+                } else if (colKey=='upgrade') {
+                    try {
+                        const r = await fetch(`/api/v1/upgradedevice?gateway=${row.gateway}&key=${row.key}`, {method: "GET"});
+                    } catch (err) {
+                    }
                 }
             });
 
@@ -817,6 +885,181 @@ class MBIOWsMonitor(MBIOTask):
         h.write(data)
         handler.wfile.write(h.bytes())
 
+    def cb_alldevices(self, handler, params):
+        handler.send_response(200)
+        handler.send_header("Content-Type", "text/html; charset=utf-8")
+        handler.end_headers()
+
+        mbio=self.getMBIO()
+        h=Html('Digimat MBIO Processor Monitor')
+        gateway=params.get('gateway')
+
+        data="""
+            <div class="page container-fluid">
+            <div class="card p-4 p-lg-5">
+            <div class="d-flex align-items-center justify-content-between flex-wrap gap-3 mb-3">
+            <div>
+            <h1 class="h3 mb-1">Digimat MBIO Processor Monitor</h1>
+            <p class="text-secondary mb-0"><a href="/gateways">Gateways</a> / <b>devices</b></p>
+            </div>
+            <div class="text-muted small">{mbio.version} /
+            <a href="/tasks">Tasks</a> /
+            <a href="/values">Values</a> /
+            <a href="/valuesmarked">Marks</a>
+            </div>
+            </div>
+            <table id="items" class="display nowrap" style="width:100%">
+            <thead>
+            <tr>
+            <th>Gateway</th>
+            <th>Address</th>
+            <th>Key</th>
+            <th>Vendor</th>
+            <th>Model</th>
+            <th>Version</th>
+            <th>Upgrade</th>
+            <th>State</th>
+            <th>Age</th>
+            <th>Error</th>
+            <th>#Values</th>
+            <th>#Msg</th>
+            <th>#MsgErr</th>
+            <th>Class</th>
+            <th>Reboot</th>
+            </tr>
+            </thead>
+            </table>
+            </div>
+            </div>
+        """
+        data=data.replace('{mbio.version}', mbio.version)
+        h.write(data)
+
+        data="""
+        <script>
+            $(function () {
+            const table = new DataTable('#items', {
+            responsive: true,
+            paging: false,
+            searching: true,
+            ordering: true,
+            info: false,
+            search: {
+                smart: true,
+                regex: false,
+                caseInsensitive: true,
+                boundary: false
+            },
+            ajax: {
+                url: "/api/v1/getalldevices",
+                dataSrc: "data"
+            },
+            columns: [
+                { data: "gateway" },
+                { data: "address" },
+                { data: "key" },
+                { data: "vendor" },
+                { data: "model" },
+                { data: "version" },
+                { data: "upgrade",
+                    render: function (data, type, row) {
+                        if (type !== "display") return data;
+                        if (row.upgrade) {
+                        return `<div class="d-flex justify-content-between align-items-center">
+                                <button class="btn btn-sm btn-outline-secondary cell-action"
+                                    data-field="upgrade" data-id="${row.key}">
+                                Upgrade
+                                </button>
+                                </div>`;
+                        }
+                        return '';
+                    }
+                },
+                { data: "state",
+                    render: function (data, type, row) {
+                        if (type !== "display") return data;
+                        return `<div class="d-flex justify-content-between align-items-center">
+                            <span>${data}</span>
+                            <button class="btn btn-sm btn-outline-secondary cell-action"
+                                data-field="state" data-id="${row.key}">
+                                Reset
+                            </button>
+                            </div>`;
+                        }
+                },
+                { data: "statetime" },
+                { data: "error", render: v=>v ? 'ERROR' : '' },
+                { data: "countvalues" },
+                { data: "countmsg" },
+                { data: "countmsgerr" },
+                { data: "class" },
+                { data: "reboot",
+                    render: function (data, type, row) {
+                        if (type !== "display") return data;
+                        if (row.reboot) {
+                        return `<div class="d-flex justify-content-between align-items-center">
+                                <button class="btn btn-sm btn-outline-secondary cell-action"
+                                    data-field="reboot" data-id="${row.key}">
+                                Reboot
+                                </button>
+                                </div>`;
+                        }
+                        return '';
+                    }
+                }
+            ],
+            rowCallback: function (row, data) {
+                if (data.error) {
+                    $('td', row).css('background-color', 'pink');
+                }
+            }
+            });
+
+            setInterval(function () {
+                table.ajax.reload(null, false);
+                }, 2000);
+
+            $("#items tbody").on("click", ".cell-action, .action-btn", async function (e) {
+                e.stopPropagation();
+
+                const $ui = $(this);
+                const $tr    = $ui.closest('tr');
+                const colKey = $ui.data('field');
+                const rowApi = table.row($tr);
+                const row    = rowApi.data();
+
+                if (colKey=='state') {
+                    try {
+                        const r = await fetch(`/api/v1/resetdevice?gateway=${row.gateway}&key=${row.key}`, {method: "GET"});
+                    } catch (err) {
+                    }
+                } else if (colKey=='reboot') {
+                    try {
+                        const r = await fetch(`/api/v1/rebootdevice?gateway=${row.gateway}&key=${row.key}`, {method: "GET"});
+                    } catch (err) {
+                    }
+                } else if (colKey=='upgrade') {
+                    try {
+                        const r = await fetch(`/api/v1/upgradedevice?gateway=${row.gateway}&key=${row.key}`, {method: "GET"});
+                    } catch (err) {
+                    }
+                }
+            });
+
+            $('#items').on('click','tr', function() {
+                var data = table.row(this).data();
+                const url = "/devicevalues?gateway=" + encodeURIComponent(data.gateway)
+                    + "&device=" + encodeURIComponent(data.key);
+                window.location.href = url;
+            });
+
+            });
+        </script>
+        """
+
+        h.write(data)
+        handler.wfile.write(h.bytes())
+
     def cb_taskvalues(self, handler, params):
         handler.send_response(200)
         handler.send_header("Content-Type", "text/html; charset=utf-8")
@@ -893,6 +1136,7 @@ class MBIOWsMonitor(MBIOTask):
                         <p class="text-secondary mb-0"><a href="/gateways">Gateways</a> / <a href="{url}">{gateway}</a> / <b>{device}</b> / values</p>
                     </div>
                     <div class="text-muted small">{mbio.version} /
+                    <a href="/alldevices">*Devices</a> /
                     <a href="/tasks">Tasks</a> /
                     <a href="/values">Values</a> /
                     <a href="/valuesmarked">Marks</a>
@@ -1355,6 +1599,7 @@ class MBIOWsMonitor(MBIOTask):
                     </div>
                     <div class="text-muted small">{mbio.version} /
                     <a href="/gateways">Gateways</a> /
+                    <a href="/alldevices">*Devices</a> /
                     <a href="/tasks">Tasks</a> /
                     <a href="/valuesmarked">Marks</a>
                     </div>
@@ -1407,6 +1652,7 @@ class MBIOWsMonitor(MBIOTask):
                         </div>
                         <div class="text-muted small">{mbio.version} /
                             <a href="/gateways">Gateways</a> /
+                            <a href="/alldevices">*Devices</a> /
                             <a href="/tasks">Tasks</a> /
                             <a href="/valuesmarked">Marks</a>
                         </div>
@@ -1459,6 +1705,7 @@ class MBIOWsMonitor(MBIOTask):
                     </div>
                     <div class="text-muted small">{mbio.version} /
                     <a href="/gateways">Gateways</a> /
+                    <a href="/alldevices">*Devices</a> /
                     <a href="/tasks">Tasks</a> /
                     <a href="/valuesmarked">Marks</a>
                     </div>
@@ -1511,6 +1758,7 @@ class MBIOWsMonitor(MBIOTask):
                     </div>
                     <div class="text-muted small">{mbio.version} /
                     <a href="/gateways">Gateways</a> /
+                    <a href="/alldevices">*Devices</a> /
                     <a href="/tasks">Tasks</a>
                     </div>
                     </div>
@@ -1554,6 +1802,7 @@ class MBIOWsMonitor(MBIOTask):
             ws.registerGetCallback('/api/v1/gettasks', self.cb_gettasks)
             ws.registerGetCallback('/api/v1/getgateways', self.cb_getgateways)
             ws.registerGetCallback('/api/v1/getgatewaydevices', self.cb_getgatewaydevices)
+            ws.registerGetCallback('/api/v1/getalldevices', self.cb_getalldevices)
             ws.registerGetCallback('/api/v1/gettaskvalues', self.cb_gettaskvalues)
             ws.registerGetCallback('/api/v1/getdevicevalues', self.cb_getdevicevalues)
             ws.registerGetCallback('/api/v1/getvalues', self.cb_getvalues)
@@ -1565,11 +1814,13 @@ class MBIOWsMonitor(MBIOTask):
             ws.registerGetCallback('/api/v1/resetgateway', self.cb_resetgateway)
             ws.registerGetCallback('/api/v1/resetdevice', self.cb_resetdevice)
             ws.registerGetCallback('/api/v1/rebootdevice', self.cb_rebootdevice)
+            ws.registerGetCallback('/api/v1/upgradedevice', self.cb_upgradedevice)
             ws.registerGetCallback('/api/v1/beep', self.cb_beep)
             ws.registerGetCallback('/test', self.cb_mycallback)
             ws.registerGetCallback('/tasks', self.cb_tasks)
             ws.registerGetCallback('/gateways', self.cb_gateways)
             ws.registerGetCallback('/devices', self.cb_devices)
+            ws.registerGetCallback('/alldevices', self.cb_alldevices)
             ws.registerGetCallback('/taskvalues', self.cb_taskvalues)
             ws.registerGetCallback('/devicevalues', self.cb_devicevalues)
             ws.registerGetCallback('/values', self.cb_values)
